#! /bin/bash

# Script to run a set of tests pow() function.
# Installed program are needed: avr-gcc, simulavr and Avr-libc.
# Possible arguments are: a list of test's (C sources).

srcdir=../src
incdir=../include
test_list=${*:-"t_pow_*.c"}

set -e

avr_gcc=${AVRGCC:-"avr-gcc"}

core=core_avr_dump.core

set_src_list ()
{
    case $1 in
	*_pow* )
	    src_list="$srcdir/pow.S $srcdir/isfinite.S -lm"
	    host_list="-lm"
	    ;;
	*)
	    src_list=
	    host_list=
	    ;;
    esac
}

measure_size ()
{
    echo "-------- size:  $*"
    bname=`basename $1 .c`
    shift
    mcu=$1
    shift
    $avr_gcc $cflags -mmcu=$mcu $* -o $bname.elf $bname.c $src_list
    avr-size $bname.elf
    rm -f $bname.elf
}

run_size ()
{
    for mcu in atmega8 ; do
	for opt in "-Os -frename-registers" ; do
	    measure_size $1 $mcu $opt
	done
    done
}

measure_speed ()
{
    echo "-------- speed:  $*"
    bname=`basename $1 .c`
    shift
    mcu=$1
    shift
    for nloops in 10 11 ; do
        $avr_gcc $cflags -mmcu=$mcu $* -DNLOOPS=$nloops \
	    -o $bname.elf $bname.c $src_list
	exit_addr=0x`avr-nm $bname.elf | grep __stop_program | cut -f1 -d' '`
        avr-objcopy -O binary $bname.elf $bname.bin
	simulavr -d $mcu -B $exit_addr -C $bname.bin | grep "clock cycles"
        if ! grep -q r24=00 $core || ! grep -q r25=00 $core
	then
	    echo >&2 "*** Simulate: exit with non-zero code"
	    exit 1
	fi
    done
    rm -f $bname.{elf,bin} $core
}

run_speed ()
{
    for mcu in at90s8515 atmega8 ; do
	for opt in "-Os -frename-registers" ; do
	    measure_speed $1 $mcu $opt
	done
    done
}

simulate ()
{
    echo "-------- simulate:  $*"
    bname=`basename $1 .c`
    shift
    mcu=$1
    shift
    $avr_gcc $cflags -mmcu=$mcu $* -o $bname.elf $bname.c $src_list
    exit_addr=0x`avr-nm $bname.elf | grep __stop_program | cut -f1 -d' '`
    avr-objcopy -O binary $bname.elf $bname.bin
    simulavr -d $mcu -B $exit_addr -C $bname.bin >/dev/null
    if ! grep -q r24=00 $core || ! grep -q r25=00 $core
    then
	echo >&2 "*** Simulate: exit with non-zero code"
	exit 1
    fi
    rm -f $bname.{elf,bin} $core
}

run_simulate ()
{
    for mcu in at90s8515 atmega8 ; do
	for opt in "-Os -frename-registers" ; do
	    simulate $1 $mcu $opt
	done
    done
}

run_host ()
{
    echo "-------- test of test: $1"
    bname=`basename $1 .c`
    cc $cflags -D TEST_OF_TEST -O2 -std=c99 \
	-o $bname.exe $bname.c $host_list
    ./$bname.exe || {
	echo >&2 "*** Run test of test: exit with $?" ;
        exit 1 ;
    }
    rm -f $bname.exe
}

set_options ()
{
    CFLAGS=
    eval `grep CFLAGS $1`
    cflags="-Wall -W -Werror -pipe $CFLAGS"
}

for f in $test_list ; do
    set_src_list $f
    set_options  $f
    run_host $f
    cflags="$cflags -I$srcdir -I$incdir"
    run_simulate $f
done

echo "$0: All OK"
# eof
